import { useCallback, useState, useMemo } from 'react';
import { scaleTime, scaleLog, extent, max, line } from 'd3';

import { XAxis } from './XAxis';
import { YAxis } from './YAxis';
import { VoronoiOverlay } from './VoronoiOverlay';

const xValue = (d) => d.date;
const yValue = (d) => d.deathTotal;

const margin = {
  top: 40,
  right: 50,
  bottom: 58,
  left: 70,
};

export const LineChart = ({ data, width, height }) => {
  const innerWidth = width - margin.right - margin.left;
  const innerHeight = height - margin.top - margin.bottom;

  const [
    activeCountryName,
    setActiveCountryName,
  ] = useState();

  const allData = useMemo(
    () =>
      data.reduce(
        (acc, countryData) => acc.concat(countryData),
        []
      ),
    [data]
  );

  const epsilon = 1;

  const xScale = useMemo(
    () =>
      scaleTime()
        .domain(extent(allData, xValue))
        .range([0, innerWidth]),
    [allData, xValue]
  );

  const yScale = useMemo(
    () =>
      scaleLog()
        .domain([epsilon, max(allData, yValue)])
        .range([innerHeight, 0]),
    [epsilon, allData, yValue]
  );

  const lineGenerator = useMemo(
    () =>
      line()
        .x((d) => xScale(xValue(d)))
        .y((d) => yScale(epsilon + yValue(d))),
    [xScale, xValue, yScale, yValue]
  );

  const now = xScale.domain()[1];

  const handleVornonHover = useCallback((d) => {
    setActiveCountryName(d.countryName);
  }, []);

  return (
    <svg width={width} height={height}>
      <g
        transform={`translate(${margin.left}, ${margin.top})`}
      >
        <XAxis xScale={xScale} innerHeight={innerHeight} />
        <YAxis yScale={yScale} innerWidth={innerWidth} />
        {data.map((countryData) => {
          return (
            <path
              className="marker-line"
              d={lineGenerator(countryData)}
            />
          );
        })}

        {activeCountryName && (
          <>
            <path
              className="marker-line active"
              d={lineGenerator(
                data.find(
                  (countryData) =>
                    countryData.countryName ===
                    activeCountryName
                )
              )}
            />
            <circle />
          </>
        )}

        <text
          transform={`translate(${innerWidth / 2}, -5)`}
          text-anchor="middle"
        >
          Coronavirus Global Deaths Over Time
        </text>
        <text
          className="axis-label"
          transform={`translate(-40,${
            innerHeight / 2
          }) rotate(-90)`}
          text-anchor="middle"
        >
          Cumulative Deaths
        </text>
        <text
          className="axis-label"
          transform={`translate(${innerWidth / 2}, ${
            innerHeight + 45
          })`}
          text-anchor="middle"
          alignment-baseline="hanging"
        >
          Time
        </text>
        <VoronoiOverlay
          onHover={handleVornonHover}
          innerWidth={innerWidth}
          innerHeight={innerHeight}
          allData={allData}
          lineGenerator={lineGenerator}
          handleVornonHover={handleVornonHover}
        />
      </g>
    </svg>
  );
};
